@@ -1,6 +1,7 @@
 import re
 from datetime import datetime
-from typing import List, Literal, Optional, Sequence
+from enum import Enum
+from typing import Any, List, Literal, Optional, Sequence
 
 from pydantic import (
     AliasChoices,
@@ -24,6 +25,7 @@ sort_fields = {
     "date": "family_publication_ts",
     "title": "family_name",
     "name": "family_name",
+    "concept_counts": "concept_counts.value",
 }
 
 filter_fields = {
@@ -38,6 +40,16 @@ _ID_ELEMENT = r"[a-zA-Z0-9]+([-_]?[a-zA-Z0-9]+)*"
 ID_PATTERN = re.compile(rf"{_ID_ELEMENT}\.{_ID_ELEMENT}\.{_ID_ELEMENT}\.{_ID_ELEMENT}")
 
 
+class OperandTypeEnum(Enum):
+    """Enumeration of possible operands for yql queries"""
+
+    GREATER_THAN = ">"
+    GREATER_THAN_OR_EQUAL = ">="
+    LESS_THAN = "<"
+    LESS_THAN_OR_EQUAL = "<="
+    EQUALS = "="
+
+
 class MetadataFilter(BaseModel):
     """A filter for metadata fields"""
 
@@ -66,57 +78,12 @@ class ConceptFilter(BaseModel):
         To get around this we query on "Q1," instead using the comma suffix to separate
         values.
         """
-        if self.name == "parent_concept_ids_flat" and self.value[-1] != ",":
-            self.value = self.value + ","
-        return self
-
-
-class Concept(BaseModel):
-    """
-    A concept extracted from a passage of text.
-
-    This refers to a span of text within passage that holds a concept.
-    E.g. "Adaptation strategy" is a concept within a passage starting at index 0 and
-    ending at index 17, classified by model "environment_model_1" on the 12th Jan at
-    12:00.
-    """
-
-    id: str
-    name: str
-    parent_concepts: List[dict[str, str]]
-    parent_concept_ids_flat: str
-    model: str
-    end: int
-    start: int
-    timestamp: datetime
-
-    model_config = ConfigDict(
-        use_enum_values=True,
-        json_encoders={datetime: lambda dt: dt.isoformat()},
-    )
-
-    @model_validator(mode="after")
-    def validate_parent_concept_ids_flat(self) -> "Concept":
-        """
-        Validate parent_concept_ids_flat field.
-
-        This field should hold the same ids as concepts in the parent_concepts field.
-        """
-        parent_concept_ids_flattened = ",".join(
-            [parent_concept["id"] for parent_concept in self.parent_concepts]
-        )
-
-        if not (
-            self.parent_concept_ids_flat == parent_concept_ids_flattened
-            or self.parent_concept_ids_flat == parent_concept_ids_flattened + ","
+        if (
+            self.name == "parent_concept_ids_flat"
+            and self.value
+            and self.value[-1] != ","
         ):
-            raise ValueError(
-                "parent_concept_ids_flat must be a comma separated list of parent "
-                "concept ids held in the parent concepts field. "
-                f"Received parent_concept_ids_flat: {self.parent_concept_ids_flat}\n"
-                "Received ids in the parent_concept objects: "
-                f"{parent_concept_ids_flattened}"
-            )
+            self.value = self.value + ","
         return self
 
 
@@ -151,6 +118,113 @@ class Filters(BaseModel):
         return clean_values
 
 
+class ConceptCountFilter(BaseModel):
+    """
+    A filter for a concept count.
+
+    Can combine filters for concept ID and concept count to achieve logic like:
+    - Documents with greater than 10 matches of concept Q123.
+    - Documents with greater than 1000 matches of any concept.
+
+    These ConceptCountFilters can be combined with an 'and' operator to create more
+    complex queries like:
+    - Documents with more than 10 matches for concept Q123 and more than 5 matches for
+        concept Q456.
+
+    param concept_id: If provided this is the ID of the concept to filter on. If it
+        left blank then all concepts that match the query will be counted.
+    param count: The number of matches to filter on.
+    param operand: The operand to use for the filter.
+        E.g. we want to filter for documents with more than 10 matches of concept Q123.
+    param negate: Whether to negate the filter.
+        E.g. we want to filter for documents that do NOT have a match for a concept.
+    """
+
+    concept_id: Optional[str] = None
+    count: int
+    operand: OperandTypeEnum
+    negate: bool = False
+
+
+class ConceptV2PassageFilter(BaseModel):
+    """
+    A filter for v2 concept data in passage spans.
+
+    Examples:
+    - Find passages with Wikibase ID Q10: ConceptV2PassageFilter(concept_wikibase_id="Q10")
+    - Find passages with concept ID "nhhzwfva": ConceptV2PassageFilter(concept_id="nhhzwfva")
+    - Find passages with classifier ID "chrtt0a7": ConceptV2PassageFilter(classifier_id="chrtt0a7")
+    """
+
+    concept_id: str | None = None
+    """If provided this is the concept ID to filter on. If left blank then all concepts that match will be counted."""
+
+    concept_wikibase_id: str | None = None
+    """If provided this is the Wikibase ID (e.g. Q374) to filter on. If left blank then all concepts that match will be counted."""
+
+    classifier_id: str | None = None
+    """If provided this is the classifier ID to filter on. If left blank then all classifiers that match will be counted."""
+
+    negate: bool = False
+    """Whether to negate the filter. E.g. we want to filter for documents that do NOT have a match for a concept."""
+
+    @model_validator(mode="after")
+    def validate_at_least_one_field(self) -> "ConceptV2PassageFilter":
+        """Ensure at least one filter field is provided."""
+        if not any([self.concept_id, self.concept_wikibase_id, self.classifier_id]):
+            raise ValueError("At least one constraint must be provided")
+        return self
+
+
+class ConceptV2DocumentFilter(BaseModel):
+    """
+    A filter for v2 concept counts in documents.
+
+    Can combine filters for concept ID, Wikibase ID, classifier ID,
+    and concept count to achieve logic like:
+    - Documents with greater than 10 matches of concept "nhhzwfva".
+    - Documents with greater than 5 matches of concept Q374.
+    - Documents with greater than 1000 matches of any concept from classifier "chrtt0a7".
+
+    These ConceptV2DocumentFilters can be combined with an 'and' operator to create more
+    complex queries like:
+    - Documents with more than 10 matches for concept "nhhzwfva" and more than 5 matches for
+      concept Q374.
+    """
+
+    concept_id: str | None = None
+    """If provided this is the concept ID to filter on. If left blank then all concepts that match will be counted."""
+
+    concept_wikibase_id: str | None = None
+    """If provided this is the Wikibase ID (e.g. Q374) to filter on. If left blank then all concepts that match will be counted."""
+
+    classifier_id: str | None = None
+    """If provided this is the classifier ID to filter on. If left blank then all classifiers that match will be counted."""
+
+    count: int | None = None
+    """The number of matches to filter on."""
+
+    operand: OperandTypeEnum | None = None
+    """The operand to use for the filter. E.g. we want to filter for documents with more than 10 matches of concept "nhhzwfva"."""
+
+    negate: bool = False
+    """Whether to negate the filter. E.g. we want to filter for documents that do NOT have a match for a concept."""
+
+    @model_validator(mode="after")
+    def validate_at_least_one_field(self) -> "ConceptV2DocumentFilter":
+        """Ensure at least one filter field is provided."""
+        if not any(
+            [
+                self.concept_id,
+                self.concept_wikibase_id,
+                self.classifier_id,
+                all([self.count, self.operand]),
+            ],
+        ):
+            raise ValueError("At least one constraint must be provided")
+        return self
+
+
 class SearchParameters(BaseModel):
     """Parameters for a search request"""
 
@@ -251,15 +325,50 @@ class SearchParameters(BaseModel):
     A field and item mapping to search in the concepts field of the document passages.
     """
 
+    custom_vespa_request_body: Optional[dict[str, Any]] = None
+    """
+    Extra fields to be added to the vespa request body. Overrides any existing fields,
+    so can also be used to override YQL or ranking profiles.
+    """
+
+    concept_count_filters: Optional[Sequence[ConceptCountFilter]] = None
+    """
+    A list of concept count filters to apply to the search.
+    """
+
+    concept_v2_passage_filters: Sequence[ConceptV2PassageFilter] | None = None
+    """
+    A list of v2 concept filters to apply to passages' spans.
+    """
+
+    concept_v2_document_filters: Sequence[ConceptV2DocumentFilter] | None = None
+    """
+    A list of v2 concept count filters to apply to documents.
+    """
+
+    replace_acronyms: bool = False
+    """
+    Whether to perform acronym replacement based on the 'acronyms' ruleset.
+    See docs: https://docs.vespa.ai/en/query-rewriting.html#rule-bases
+    """
+
+    distance_threshold: Optional[float] = 0.24
+    """
+    Optional threshold for the nearest neighbor search distance. Results with a
+    distance score below this threshold will be excluded. Based on the 'innerproduct'
+    distance metric, lower scores are less relevant.
+    """
+
+    by_document_title: bool = False
+    """
+    Whether to search by document title rather than family title.
+    """
+
     @model_validator(mode="after")
     def validate(self):
         """Validate against mutually exclusive fields"""
         if self.exact_match and self.all_results:
             raise ValueError("`exact_match` and `all_results` are mutually exclusive")
-        if self.documents_only and not self.all_results:
-            raise ValueError(
-                "`documents_only` requires `all_results`, other queries are not supported"
-            )
         return self
 
     @model_validator(mode="after")
@@ -377,10 +486,10 @@ class Hit(BaseModel):
     document_content_type: Optional[str] = None
     document_cdn_object: Optional[str] = None
     document_source_url: Optional[str] = None
+    document_title: Optional[str] = None
     corpus_type_name: Optional[str] = None
     corpus_import_id: Optional[str] = None
     metadata: Optional[Sequence[dict[str, str]]] = None
-    concepts: Optional[Sequence[Concept]] = None
     relevance: Optional[float] = None
     rank_features: Optional[dict[str, float]] = None
 
@@ -408,10 +517,27 @@ class Hit(BaseModel):
             raise ValueError(f"Unknown response type: {response_type}")
         return hit
 
+    def __eq__(self, other):
+        """
+        Check if two hits are equal.
+
+        Ignores relevance and rank_features as these are dependent on non-deterministic query routing.
+        """
+        if not isinstance(other, self.__class__):
+            return False
+
+        fields_to_compare = [
+            f for f in self.__dict__.keys() if f not in ("relevance", "rank_features")
+        ]
+
+        return all(getattr(self, f) == getattr(other, f) for f in fields_to_compare)
+
 
 class Document(Hit):
     """A document search result hit."""
 
+    concept_counts: Optional[dict[str, int]] = None
+
     @classmethod
     def from_vespa_response(cls, response_hit: dict) -> "Document":
         """
@@ -443,23 +569,77 @@ class Document(Hit):
             document_content_type=fields.get("document_content_type"),
             document_cdn_object=fields.get("document_cdn_object"),
             document_source_url=fields.get("document_source_url"),
+            document_title=fields.get("document_title"),
             corpus_type_name=fields.get("corpus_type_name"),
             corpus_import_id=fields.get("corpus_import_id"),
             metadata=fields.get("metadata"),
-            concepts=fields.get("concepts"),
             relevance=response_hit.get("relevance"),
             rank_features=fields.get("summaryfeatures"),
+            concept_counts=fields.get("concept_counts"),
         )
 
 
 class Passage(Hit):
     """A passage search result hit."""
 
+    class Concept(BaseModel):
+        """
+        A concept extracted from a passage of text.
+
+        This refers to a span of text within passage that holds a concept.
+        E.g. "Adaptation strategy" is a concept within a passage starting at index 0 and
+        ending at index 17, classified by model "environment_model_1" on the 12th Jan at
+        12:00.
+        """
+
+        id: str
+        name: str
+        parent_concepts: Optional[List[dict[str, str]]] = None
+        parent_concept_ids_flat: Optional[str] = None
+        model: str
+        end: int
+        start: int
+        timestamp: datetime
+
+        model_config = ConfigDict(
+            use_enum_values=True,
+            json_encoders={datetime: lambda dt: dt.isoformat()},
+        )
+
+        @model_validator(mode="after")
+        def validate_parent_concept_ids_flat(self) -> "Passage.Concept":
+            """
+            Validate parent_concept_ids_flat field.
+
+            This field should hold the same ids as concepts in the parent_concepts field.
+            """
+            # Skip validation if either field is missing
+            if self.parent_concepts is None or self.parent_concept_ids_flat is None:
+                return self
+
+            parent_concept_ids_flattened = ",".join(
+                [parent_concept["id"] for parent_concept in self.parent_concepts]
+            )
+
+            if not (
+                self.parent_concept_ids_flat == parent_concept_ids_flattened
+                or self.parent_concept_ids_flat == parent_concept_ids_flattened + ","
+            ):
+                raise ValueError(
+                    "parent_concept_ids_flat must be a comma separated list of parent "
+                    "concept ids held in the parent concepts field. "
+                    f"Received parent_concept_ids_flat: {self.parent_concept_ids_flat}\n"
+                    "Received ids in the parent_concept objects: "
+                    f"{parent_concept_ids_flattened}"
+                )
+            return self
+
     text_block: str
     text_block_id: str
     text_block_type: str
     text_block_page: Optional[int] = None
     text_block_coords: Optional[Sequence[tuple[float, float]]] = None
+    concepts: Optional[Sequence[Concept]] = None
 
     @classmethod
     def from_vespa_response(cls, response_hit: dict) -> "Passage":
@@ -517,6 +697,20 @@ class Family(BaseModel):
     prev_continuation_token: Optional[str] = None
     relevance: Optional[float] = None
 
+    def __eq__(self, other):
+        """
+        Check if two Families are equal.
+
+        Ignores relevance as it's dependent on non-deterministic query routing.
+        """
+
+        if not isinstance(other, self.__class__):
+            return False
+
+        fields_to_compare = [f for f in self.__dict__.keys() if f not in ("relevance")]
+
+        return all(getattr(self, f) == getattr(other, f) for f in fields_to_compare)
+
 
 class SearchResponse(BaseModel):
     """Relevant results, and search response metadata"""
@@ -529,3 +723,21 @@ class SearchResponse(BaseModel):
     continuation_token: Optional[str] = None
     this_continuation_token: Optional[str] = None
     prev_continuation_token: Optional[str] = None
+
+    def __eq__(self, other):
+        """
+        Check if two hits are equal.
+
+        Ignores query time fields as they are non-deterministic.
+        """
+
+        if not isinstance(other, self.__class__):
+            return False
+
+        fields_to_compare = [
+            f
+            for f in self.__dict__.keys()
+            if f not in ("query_time_ms", "total_time_ms")
+        ]
+
+        return all(getattr(self, f) == getattr(other, f) for f in fields_to_compare)
