@@ -8,7 +8,6 @@ from typing import Any, Final, List, Optional, Sequence, Tuple, TypeVar, Union
 
 from cpr_sdk.pipeline_general_models import (
     CONTENT_TYPE_HTML,
-    CONTENT_TYPE_PDF,
     BackendDocument,
     Json,
 )
@@ -72,9 +71,9 @@ class TextBlock(BaseModel):
 
     text: List[str]
     text_block_id: str
-    language: Optional[
-        str
-    ] = None  # TODO: validate this against a list of language ISO codes
+    language: Optional[str] = (
+        None  # TODO: validate this against a list of language ISO codes
+    )
     type: BlockType
     type_confidence: float = Field(ge=0, le=1)
 
@@ -196,24 +195,15 @@ class BaseParserOutput(BaseModel):
         """
         Validate the relationship between content-type and the data that is set.
 
-        Check that html_data is set if content_type is HTML, or pdf_data is set if
-        content_type is PDF.
-
         Check that if the content-type is not HTML or PDF, then html_data and pdf_data
         are both null.
         """
-        if self.document_content_type == CONTENT_TYPE_HTML and self.html_data is None:
-            raise ValueError("html_data must be set for HTML documents")
 
-        if self.document_content_type == CONTENT_TYPE_PDF and self.pdf_data is None:
-            raise ValueError("pdf_data must be set for PDF documents")
+        document_has_data = self.html_data is not None or self.pdf_data is not None
 
-        if self.document_content_type not in {
-            CONTENT_TYPE_HTML,
-            CONTENT_TYPE_PDF,
-        } and (self.html_data is not None or self.pdf_data is not None):
+        if not self.document_content_type and document_has_data:
             raise ValueError(
-                "html_data and pdf_data must be null for documents with no content type."
+                "html_data or pdf_data must be null for documents with no content type."
             )
 
         return self
@@ -234,12 +224,11 @@ class BaseParserOutput(BaseModel):
 
         :return: Sequence[TextBlock]
         """
-        if self.document_content_type == CONTENT_TYPE_HTML:
-            html_data: Union[HTMLData, None] = self.html_data
-            return html_data.text_blocks if html_data else []
-        elif self.document_content_type == CONTENT_TYPE_PDF:
-            pdf_data: Union[PDFData, None] = self.pdf_data
-            return pdf_data.text_blocks if pdf_data else []
+
+        if self.html_data is not None:
+            return self.html_data.text_blocks
+        elif self.pdf_data is not None:
+            return self.pdf_data.text_blocks
         return []
 
     def to_string(self) -> str:  # type: ignore
@@ -380,7 +369,7 @@ class ParserOutput(BaseParserOutput):
 
     @staticmethod
     def _rename_text_block_keys(
-        keys: Union[list[str], dict[str, Any]]
+        keys: Union[list[str], dict[str, Any]],
     ) -> Union[list[str], dict[str, Any]]:
         """Prepend text_block. to the keys in the dictionary or list."""
 
